==> python-language-24.2.0-sources.jar: com/oracle/graal/python/builtins/objects/contextvars/PContextVar.java <==
/*
 * Copyright (c) 2022, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.graal.python.builtins.objects.contextvars;

import com.oracle.graal.python.builtins.objects.object.PythonBuiltinObject;
import com.oracle.graal.python.runtime.PythonContext;
import com.oracle.truffle.api.object.Shape;
import com.oracle.truffle.api.strings.TruffleString;

public final class PContextVar extends PythonBuiltinObject {
    private static int nextId = 0;
    private final int hashForHamt = nextId++;
    private final TruffleString name;
    private final Object def;

    public static final Object NO_DEFAULT = new Object();

    public PContextVar(Object cls, Shape instanceShape, TruffleString name, Object def) {
        super(cls, instanceShape);
        this.name = name;
        this.def = def;
    }

    public int getHash() {
        return hashForHamt * (hashForHamt + 3);
    }

    public TruffleString getName() {
        return name;
    }

    public Object getDefault() {
        return def;
    }

    public Object getValue(PythonContext.PythonThreadState state) {
        return state.getContextVarsContext().contextVarValues.lookup(this, getHash());
    }

    public void setValue(PythonContext.PythonThreadState state, Object value) {
        PContextVarsContext current = state.getContextVarsContext();
        current.contextVarValues = current.contextVarValues.withEntry(new Hamt.Entry(this, getHash(), value));
    }

    public Object get(PythonContext.PythonThreadState state, Object defaultValue) {
        Object result = getValue(state);
        if (result != null) {
            return result;
        }
        if (defaultValue != PContextVar.NO_DEFAULT) {
            return defaultValue;
        }
        if (def != PContextVar.NO_DEFAULT) {
            return def;
        }
        return null;
    }
}

==> python-language-25.0.0-sources.jar: com/oracle/graal/python/builtins/objects/contextvars/PContextVar.java <==
/*
 * Copyright (c) 2022, 2025, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.graal.python.builtins.objects.contextvars;

import com.oracle.graal.python.builtins.objects.object.PythonBuiltinObject;
import com.oracle.graal.python.runtime.PythonContext;
import com.oracle.truffle.api.nodes.Node;
import com.oracle.truffle.api.object.Shape;
import com.oracle.truffle.api.strings.TruffleString;

public final class PContextVar extends PythonBuiltinObject {
    private static int nextId = 0;
    private final int hashForHamt = nextId++;
    private final TruffleString name;
    private final Object def;

    public static final Object NO_DEFAULT = new Object();

    public PContextVar(Object cls, Shape instanceShape, TruffleString name, Object def) {
        super(cls, instanceShape);
        this.name = name;
        this.def = def;
    }

    public int getHash() {
        return hashForHamt * (hashForHamt + 3);
    }

    public TruffleString getName() {
        return name;
    }

    public Object getDefault() {
        return def;
    }

    public Object getValue(Node node, PythonContext.PythonThreadState state) {
        return state.getContextVarsContext(node).contextVarValues.lookup(this, getHash());
    }

    public void setValue(Node node, PythonContext.PythonThreadState state, Object value) {
        PContextVarsContext current = state.getContextVarsContext(node);
        current.contextVarValues = current.contextVarValues.withEntry(new Hamt.Entry(this, getHash(), value));
    }

    public Object get(Node node, PythonContext.PythonThreadState state, Object defaultValue) {
        Object result = getValue(node, state);
        if (result != null) {
            return result;
        }
        if (defaultValue != PContextVar.NO_DEFAULT) {
            return defaultValue;
        }
        if (def != PContextVar.NO_DEFAULT) {
            return def;
        }
        return null;
    }
}
